import 'dart:convert';
import '../models/user_model.dart';
import '../service/constaints.dart';
import 'package:sharing/remote/errors/exeptions.dart';
import 'package:injectable/injectable.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../models/headers_model.dart';

abstract class SharePrefSource {
  Future<void> cacheHeader({required HeadersModel? model});
  Future<void> cacheUser({required UserModel? data});
  Future<void> cacheAuthToken({required String? authToken});
  Future<void> cacheNumberBadge({required int number});
  Future<void> resetAllCache();
  HeadersModel? getHeader();
  UserModel? getUser();
  String? getAuthToken();
  Future<void> clearPrefs();
  Future<bool> checkAuthenticatedUser();
}

@LazySingleton(as: SharePrefSource)
class SharePrefSourceImpl implements SharePrefSource {
  final SharedPreferences _prefs;

  SharePrefSourceImpl(this._prefs);

  @override
  Future<void> clearPrefs() {
    try {
      return _prefs.clear();
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  Future<void> cacheHeader({required HeadersModel? model}) async {
    try {
      if (model != null) {
        _prefs.setString(Constaints.HEADER_CACHE, jsonEncode(model.toJson()));
      }
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  HeadersModel? getHeaders() {
    try {
      String? header = _prefs.getString(Constaints.HEADER_CACHE);
      if (header?.isNotEmpty == true) {
        return HeadersModel.fromJson(json.decode(header!));
      } else {
        return null;
      }
    } catch (e) {
      print(e.toString());
      return null;
    }
  }

  @override
  Future<bool> checkAuthenticatedUser() async {
    try {
      String? authToken = _prefs.getString(Constaints.AUTH_TOKEN_CACHE);
      if (authToken?.isNotEmpty == true) {
        return Future.value(true);
      } else {
        return Future.value(false);
      }
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  Future<void> cacheAuthToken({required String? authToken}) async {
    try {
      if (authToken?.isNotEmpty == true) {
        await _prefs.setString(Constaints.AUTH_TOKEN_CACHE, authToken!);
      }
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  String? getAuthToken() {
    try {
      String? authToken = _prefs.getString(Constaints.AUTH_TOKEN_CACHE);
      if (authToken?.isNotEmpty == true) {
        return authToken!;
      } else {
        return "";
      }
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  Future<void> resetAllCache() async {
    try {
      await _prefs.setString(Constaints.AUTH_TOKEN_CACHE, "");
      await _prefs.setString(Constaints.HEADER_CACHE, "");
      await _prefs.setString(Constaints.USER_CACHE, "");
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  Future<void> cacheNumberBadge({required int number}) async {
    try {
      await _prefs.setInt(Constaints.NUMBER_BADGE_CACHE, number);
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  HeadersModel? getHeader() {
    try {
      String? header = _prefs.getString(Constaints.HEADER_CACHE);
      if (header?.isNotEmpty == true) {
        return HeadersModel.fromJson(json.decode(header!));
      } else {
        return null;
      }
    } catch (e) {
      print(e.toString());
      return null;
    }
  }

  @override
  Future<void> cacheUser({required UserModel? data}) async {
    try {
      if (data != null) {
        _prefs.setString(Constaints.USER_CACHE, jsonEncode(data.toJson()));
      }
    } catch (e) {
      print(e.toString());
      throw CacheException();
    }
  }

  @override
  UserModel? getUser() {
    try {
      String? data = _prefs.getString(Constaints.USER_CACHE);
      if (data?.isNotEmpty == true) {
        return UserModel.fromJson(json.decode(data!));
      } else {
        return null;
      }
    } catch (e) {
      print(e.toString());
      return null;
    }
  }
  
}
