import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:easy_localization/src/public_ext.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sharing/remote/constants.dart';
import 'package:sharing/remote/network/either.dart';
import 'package:sharing/remote/network/fetch_failure.dart';

import 'base_event.dart';
import 'base_state.dart';

abstract class BaseBloc extends Bloc<BaseEvent, BaseState> {
  BaseBloc() : super(InitState()) {
    onMapEventToState();
  }
  void onMapEventToState();

  void dispatchEvent(BaseEvent event) {
    print('dispatchEvent event $event');
    super.add(event);
  }
}

Future<BaseState?> safeServerMessageState(
    {required Either<FetchFailure, String?> response,
    bool? isReturn,
    bool? isCallback}) async {
  try {
    if (response.get != null) {
      if (isCallback != null) {
        return await CallBackState(msg: response.get);
      } else {
        return await SuccessDialogState(
            message: response.get, isReturn: isReturn);
      }
    } else {
      return await callBackState(response: response);
    }
  } catch (e) {
    final failure = returnFailure(e);
    Future.value(CrashError(100, failure, "Error"));
  }
  return null;
}

Future<BaseState?> safeServerListState<T>(
    {required Either<FetchFailure, List<T>?> response,
    int? index,
    bool? isShowUiEmpty = true,
    bool? isReturn = false}) async {
  try {
    List<T>? data = [];
    if (response.get != null) {
      return await SuccessDataState(data: response.get, index: index);
    } else if (response.leftOrCrash == NoData()) {
      return await SuccessDataState(data: data);
    } else {
      return await callBackState(
          response: response, isShowUiEmpty: isShowUiEmpty, isReturn: isReturn);
    }
  } catch (e) {
    final failure = returnFailure(e);
    Future.value(CrashError(100, failure, "Error"));
  }
  return null;
}

Future<BaseState?> safeServerObjectState<T>(
    {required Either<FetchFailure, T?> response,
    bool? isShowUiEmpty = true,
    bool? isReturnPage = false,
    bool? isCallback = false}) async {
  try {
    if (response.get != null) {
      return await SuccessDataState(data: response.get);
    } else {
      return await callBackState(
          response: response,
          isShowUiEmpty: isShowUiEmpty,
          isReturn: isReturnPage,
          isCallbackState: isCallback);
    }
  } catch (e) {
    final failure = returnFailure(e);
    Future.value(CrashError(100, failure, "Error"));
  }
  return null;
}

Future<BaseState?> safeCacheState<T>(T? response) async {
  try {
    if (response != null) {
      return await SuccessDataState(data: response);
    }
  } catch (e) {
    final failure = returnFailure(e);
    Future.value(CrashError(100, failure, "Error"));
  }
  return null;
}

BaseState safeErrorState<T>(
    {required FetchFailure error,
    bool? isShowUiEmpty = true,
    bool? isReturn = false,
    bool? isCallbackState = false,
    bool? isData = false}) {
  if (error is ServerError) {
    return ServerErrorState(
        code: error.code, data: error.data, msg: error.msg, isReturn: isReturn);
  } else if (error is WarningError) {
    if (isCallbackState == true) {
      return WarningBackState(data: error.data, msg: error.msg);
    } else {
      if (isData == true) {
        return WarningState(
            code: error.code,
            data: error.data,
            msg: [error.data]);
      } else {
        return WarningState(
            code: error.code,
            data: error.data,
            msg: error.msg,
            isReturn: isReturn);
      }
    }
  } else if (error is UnknownError) {
    return UnknowErrorState(code: error.code, data: error.data, msg: error.msg);
  } else if (error is NetworkError) {
    return NetwordErrorState();
  } else if (error is CrashError) {
    return WarningState(code: error.code, data: error.data, msg: error.msg);
  } else if (error is Login) {
    if (isCallbackState == true) {
      return LoginCallBackState(data: error.data, msg: error.msg);
    } else {
      return LoginState(data: error.data, msg: error.msg);
    }
  } else if (error is Nothing) {
    return NothingState();
  } else {
    if (isShowUiEmpty == true) {
      return NoDataState(msg: "text_no_data".tr());
    } else {
      return NothingState();
    }
  }
}

Future<BaseState> callBackState<T>(
        {required Either<FetchFailure, T?> response,
        bool? isShowUiEmpty = true,
        bool? isReturn = false,
        bool? isCallbackState = false}) async =>
    response.leftOrCrash.map(
      serverError: (value) {
        return ServerErrorState(
            code: value.code,
            data: value.data,
            msg: value.msg,
            isReturn: isReturn);
      },
      networkError: (value) {
        return NetwordErrorState();
      },
      unknownError: (value) {
        return UnknowErrorState(
            code: value.code, data: value.data, msg: value.msg);
      },
      noData: (value) {
        if (isShowUiEmpty == true) {
          return NoDataState(msg: "text_no_data".tr());
        } else {
          return NothingState();
        }
      },
      login: (value) {
        if (isCallbackState == true) {
          return LoginCallBackState(data: value.data, msg: value.msg);
        } else {
          return LoginState(data: value.data, msg: value.msg);
        }
      },
      warningError: (value) {
        if (isCallbackState == true) {
          return WarningBackState(data: value.data, msg: value.msg);
        } else {
          return WarningState(
              code: value.code,
              data: value.data,
              msg: value.msg,
              isReturn: isReturn);
        }
      },
      crashError: (CrashError value) {
        return WarningState(code: value.code, data: value.data, msg: value.msg);
      },
      nothing: (Nothing value) {
        return NothingState();
      },
    );
