import 'dart:async';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_animated_dialog/flutter_animated_dialog.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sharing/common/utils/helper/animation_helper.dart';
import 'package:sharing/common/values/colors.dart';
import 'package:sharing/common/values/sizes.dart';
import 'package:sharing/widget/custom/custom_icon.dart';
import 'package:sharing/widget/dialog/custom_dialog_success.dart';
import 'package:sharing/widget/dialog/custom_dialog_warning_one_button.dart';
import 'package:sharing/widget/pages/page_empty_data_widget.dart';
import 'package:sharing/widget/pages/page_loading_ui_widget.dart';
import 'package:sharing/widget/pages/page_loading_widget.dart';
import 'package:sharing/widget/pages/page_error_widget.dart';
import 'base_bloc.dart';
import 'base_state.dart';

// ignore: must_be_immutable
class BaseBlocConsumerWidget<B extends BaseBloc> extends StatefulWidget {
  B? bloc;

  final String? messageItemEmpty;
  final BlocWidgetBuilder<BaseState>? builder;
  final BlocWidgetListener<BaseState>? listener;
  final BlocBuilderCondition<BaseState>? buildWhen;
  final BlocListenerCondition<BaseState>? listenWhen;
  final List<Type>? listenIf;
  final List<Type>? buildIf;
  final Function()? retryCallback;
  final bool? isRetryCallback;

  BaseBlocConsumerWidget(
      {Key? key,
      required this.bloc,
      this.messageItemEmpty,
      this.builder,
      this.listener,
      this.buildWhen,
      this.listenWhen,
      this.listenIf,
      this.buildIf,
      this.retryCallback,
      this.isRetryCallback})
      : super(key: key) {
    listenIf?.addAll([LoadingUiState]);
    buildIf?.addAll(
        [LoadingState, NoDataState, ServerErrorState, ServerErrorState]);
  }

  @override
  State<BaseBlocConsumerWidget> createState() => _BaseBlocConsumerWidgetState();
}

class _BaseBlocConsumerWidgetState<B extends BaseBloc>
    extends State<BaseBlocConsumerWidget> {
  Timer? _timer;
  bool? isLoading = false;
  bool? isDialog = false;
  B? bloc;

  @override
  void initState() {
    bloc = widget.bloc as B?;
    super.initState();
  }

  @override
  void didChangeDependencies() {
    if (isLoading == true) {
      dismissLoading();
    }
    super.didChangeDependencies();
  }

  @override
  void dispose() {
    _timer?.cancel();
    _timer = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    widget.bloc ??= BlocProvider.of<B>(context);
    return BlocConsumer<B, BaseState>(
      bloc: bloc,
      listenWhen: (oldState, newState) {
        if (widget.listenWhen != null) {
          return widget.listenWhen!(oldState, newState);
        }
        if (widget.listenIf != null) {
          return widget.listenIf!.any((value) => value == newState.runtimeType);
        }
        return true;
      },
      listener: (context, state) async {
        if (state is CloseLoadingState) {
          debugPrint("|==============> isLoading == Close <===============|");
          dissmisDialog();
        } else if (state is LoadingState && isLoading == false) {
          debugPrint("|==============> Create LoadingState <==============|");
          // dissmisDialog();
          customDialogShowLoading(state);
        } else if (state is ServerErrorState) {
          dissmisDialog();
          debugPrint("|==============> Create ServerErrorState <==========|");
          customDialogServerError(state);
        } else if (state is WarningState) {
          dissmisDialog();
          debugPrint("|==============> Create WarningState <==============|");
          customDialogWaring(state);
        } else if (state is UnknowErrorState) {
          dissmisDialog();
          debugPrint("|==============> Create UnknowErrorState <==========|");
          customDialogUnknownError(state);
        } else if (state is LoginState) {
          dissmisDialog();
          debugPrint("|==============> Create LoginState <================|");
          customDialogLogin(state);
        } else if (state is NetwordErrorState) {
          debugPrint("|==============> Create NetwordErrorState <========|");
          customDialogNetworkError();
        } else if (state is SuccessDialogState) {
          dissmisDialog();
          debugPrint("|==============> Create SuccessDialogState <========|");
          customDialogSuccess(state);
        }
        widget.listener?.call(context, state);
      },
      buildWhen: (oldState, newState) {
        if (widget.buildWhen != null) {
          return widget.buildWhen!(oldState, newState);
        }
        if (widget.buildIf != null) {
          return widget.buildIf!.any((value) => value == newState.runtimeType);
        }
        return true;
      },
      builder: (context, state) {
        if (widget.builder != null) {
          if (state is LoadingUiState) {
            dissmisDialog();
            return LoadingUIPageWidget(
                isCart: false, msg: widget.messageItemEmpty);
          } else if (state is NoDataState) {
            dissmisDialog();
            return EmptyUIPageWidget(
              msg: widget.messageItemEmpty ?? "text_no_data".tr(),
            );
          } else if (state is ErrorState) {
            dissmisDialog();
            return ErrorUIPageWidget(onTap: () {});
          } else {
            return widget.builder!(context, state);
          }
        } else {
          return Container();
        }
      },
    );
  }

  void timerLoading(BuildContext context, int? duration) {
    debugPrint("|==============> Create Timer <==============|");
    _timer?.cancel();
    _timer = Timer(Duration(seconds: duration ?? 0), () {
      debugPrint("|==============> End Timer <==============|");
      dissmisDialog();
    });
  }

  void dissmisDialog({bool forceDismiss = false}) {
    if (forceDismiss == true) {
      dismissDialog();
      return;
    }
    if (isLoading == true) {
      dismissLoading();
    }
    if (isDialog == true) {
      dismissDialog();
    }
  }

  void dismissLoading() {
    _timer?.cancel();
    isLoading = false;
    Navigator.of(context, rootNavigator: true).pop();
    debugPrint("|==============> Dissmis Loading **** <==============|");
  }

  void dismissDialog() {
    Navigator.of(context, rootNavigator: true).pop();
    isDialog = false;
    debugPrint("|==============> Dismiss Dialog **** <==============|");
  }

  /* region ========================= DIALOG LOADING =========================*/
  void customDialogShowLoading(LoadingState state) {
    isLoading = true;
    if (state.isShow == false) {
      timerLoading(context, state.duration);
    } else {
      timerLoading(context, 16);
    }
    showDialog(
        barrierColor: Colors.white.withOpacity(0.5),
        context: context,
        barrierDismissible: true,
        builder: (BuildContext context) {
          debugPrint("|==============> isLoading == true <==============|");
          return LoadingPageWidget(msg: state.msg);
        }).then((value) {
      debugPrint("|==============> isLoading == false --- <==============|");
      isLoading = false;
      _timer?.cancel();
    });
  }
  /* #endregion ==============================================================*/

  // ============================== DIALOG WARING ==============================
  void customDialogWaring(WarningState state) {
    showAnimatedDialog(
      context: context,
      barrierDismissible: true,
      builder: (BuildContext context) {
        isDialog = true;
        debugPrint("|==============> isDialog == true <==============|");
        return CustomDialogOne(
            content: state.msg,
            title: "text_warning".tr(),
            colorWarning: SharingColors.WarningColor,
            negativeBtnText: "text_ok".tr(),
            iconWarning: CustomIcon(
                icon: Icons.warning,
                color: Colors.white,
                size: Sizes.ICON_SIZE_24),
            isClose: false,
            positiveBtnPressed: () {
              dissmisDialog(forceDismiss: true);
              debugPrint("|==============> isDialog == false <==============|");
            });
      },
      animationType: DialogTransitionType.fadeScale,
      curve: Curves.fastOutSlowIn,
      duration: const Duration(milliseconds: 100),
    ).then((value) {
      if (state.isReturn == true) dissmisDialog();
      isDialog = false;
      debugPrint("|==============> isDialog == false <==============|");
      return true;
    });
  }
  // ===========================================================================

  // ============================== DIALOG SERVER ERROR ========================
  void customDialogServerError(ServerErrorState state) {
    showAnimatedDialog(
      context: context,
      barrierDismissible: true,
      builder: (BuildContext context) {
        isDialog = true;
        debugPrint("|==============> isDialog == true <==============|");
        return CustomDialogOne(
            content: state.msg,
            title: "text_warning".tr(),
            negativeBtnText: "text_ok".tr(),
            positiveBtnPressed: () {
              dissmisDialog(forceDismiss: true);
              debugPrint("|==============> isDialog == false <==============|");
            },
            colorWarning: SharingColors.ErrorColor,
            iconWarning: CustomIcon(
                icon: Icons.error,
                color: Colors.white,
                size: Sizes.ICON_SIZE_24));
      },
      animationType: DialogTransitionType.fadeScale,
      curve: Curves.fastOutSlowIn,
      duration: const Duration(milliseconds: 100),
    ).then((value) {
      if (state.isReturn == true) dissmisDialog();
      isDialog = false;
      debugPrint("|==============> isDialog == false <==============|");
      return true;
    });
  }
  // ===========================================================================

  // ============================== DIALOG UNKNOWN ERROR =======================
  void customDialogUnknownError(UnknowErrorState state) {
    showAnimatedDialog(
      context: context,
      barrierDismissible: true,
      builder: (BuildContext context) {
        isDialog = true;
        debugPrint("|==============> isDialog == true <==============|");
        return CustomDialogOne(
          content: [(state.code.toString() + " " + state.msg)],
          title: "text_warning".tr(),
          negativeBtnText: "text_cancel".tr(),
          positiveBtnPressed: () {
            dissmisDialog(forceDismiss: true);
            debugPrint("|==============> isDialog == false <==============|");
          },
          iconWarning: CustomIcon(
              icon: Icons.warning,
              color: Colors.white,
              size: Sizes.ICON_SIZE_24),
        );
      },
      animationType: DialogTransitionType.fadeScale,
      curve: Curves.fastOutSlowIn,
      duration: const Duration(milliseconds: 100),
    );
  }
  // ===========================================================================

  // ============================== DIALOG NETWORK ERROR =======================
  void customDialogNetworkError() {
    Future.delayed(const Duration(milliseconds: 600), () {
      AnimationHelper.CustomNetworkWarningToast(
          context: context,
          isRetryCallback: widget.isRetryCallback,
          retryCallbak: widget.retryCallback);
    });
  }
  // ===========================================================================

  // ============================== DIALOG SUCCESS =============================
  void customDialogSuccess(SuccessDialogState state) {
    showAnimatedDialog(
      context: context,
      barrierDismissible: true,
      builder: (BuildContext context) {
        isDialog = true;
        debugPrint("|==============> isDialog == true <==============|");
        return CustomDialogSuccess(
          content: state.message,
          title: "text_success".tr(),
          positiveBtnText: 'text_ok'.tr(),
          positiveBtnPressed: () {
            Navigator.of(context).pop();
            // dissmisDialog(context, forceDismiss: true);
            debugPrint("|==============> isDialog == false <==============|");
          },
        );
      },
      animationType: DialogTransitionType.fadeScale,
      curve: Curves.fastOutSlowIn,
      duration: const Duration(milliseconds: 100),
    ).then((value) {
      if (state.isReturn == true) dissmisDialog();
      debugPrint("|==============> isDialog == false <==============|");
      isDialog = false;
      return true;
    });
  }
  // ===========================================================================

  // ============================== DIALOG LOGIN =============================
  void customDialogLogin(LoginState state) async {
    debugPrint("|==============> Create LoginState <==============|");
    await showAnimatedDialog(
      context: context,
      barrierDismissible: true,
      builder: (BuildContext context) {
        isDialog = true;
        debugPrint("|==============> isDialog == true <==============|");
        return CustomDialogOne(
          content: state.msg ?? ["text_force_login".tr()],
          title: "text_request_sign_in".tr(),
          isClose: false,
          negativeBtnText: "text_ok".tr(),
          positiveBtnPressed: () {
            dissmisDialog(forceDismiss: true);
            debugPrint("|==============> isDialog == false <==============|");
          },
          iconWarning: CustomIcon(
              icon: Icons.warning,
              color: Colors.white,
              size: Sizes.ICON_SIZE_24),
        );
      },
      animationType: DialogTransitionType.fadeScale,
      curve: Curves.fastOutSlowIn,
      duration: const Duration(milliseconds: 100),
    );
  }
  // ===========================================================================
}
