import 'dart:async';

import 'package:fcm_config/fcm_config.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:pull_to_refresh_flutter3/pull_to_refresh_flutter3.dart';
import 'package:scroll_to_index/scroll_to_index.dart';
import 'package:sharing/widget/dialog/custom_will_pop_back.dart';

import '../di/injector.dart';
import '../theme/colors.dart';
import 'base_bloc.dart';
import 'base_state.dart';
import 'base_bloc_consumer_widget.dart';
import 'mixin/base_dialog_mixin.dart';
import 'mixin/base_handle_mixin.dart';
import 'mixin/connect_internet_mixin.dart';
import 'mixin/page_refresh_change_mixin.dart';

abstract class BasePage extends StatefulWidget {}

abstract class BasePageState<B extends BaseBloc> extends State<BasePage>
    with
        BaseDialogMixin,
        BaseHandleMixin,
        PageRefreshChangeMixin,
        ConnectInternetMixin,
        FCMNotificationMixin {
  B? bloc;

  RefreshController _refreshCtr = RefreshController(initialRefresh: false);
  GlobalKey _refresherKey = GlobalKey();
  AutoScrollController? controller = AutoScrollController(axis: Axis.vertical);

  @override
  Widget build(BuildContext context) {
    if (bloc == null) {
      bloc = BlocProvider.of<B>(context);
    }
    return WillPopScope(
      onWillPop: () async => await onHanderWillPop(context),
      child: AnnotatedRegion<SystemUiOverlayStyle>(
          value: SystemUiOverlayStyle(
              statusBarColor: Colors.transparent,
              systemNavigationBarColor: isBottomBarThemeColor == true
                  ? Theme.of(context).primaryColor
                  : null),
          child: Scaffold(
              resizeToAvoidBottomInset: isResizeToAvoidBottomInset,
              backgroundColor: scaffoldBackgroundColor,
              body: GestureDetector(
                  onVerticalDragDown: (details) => clearFocus(),
                  onTapDown: (value) => clearFocus(),
                  child: isFilter == true
                      ? ContainerFilter()
                      : ContainerDefault()))),
    );
  }

  Future<bool> onHanderWillPop(BuildContext context) async {
    if (isFromHome == true) {
      return false;
    } else if (isPrimary == true) {
      if (isExit == true) {
        return await CustomWillPopBack.Pop(context, true);
      } else {
        return false;
      }
    } else if (isForceExit == true) {
      SystemNavigator.pop();
      return true;
    } else {
      return true;
    }
  }

  Widget ContainerFilter() {
    return Column(
      children: [
        Expanded(
          child: ListView(
            children: [
              appTopBarMenu(context),
              isRefreshLoading == true
                  ? baseBodyWidget()
                  : baseBodyWidget(isChange: false)
            ],
          ),
        ),
        SafeArea(child: appBottomBarMenu(context), top: false),
      ],
    );
  }

  Widget ContainerDefault() {
    return Column(
      children: [
        appTopBarMenu(context),
        Expanded(
            child: Stack(
          children: [
            isRefreshLoading == true
                ? baseBodyWidget()
                : baseBodyWidget(isChange: false),
            showConnectInternet()
          ],
        )),
        SafeArea(child: appBottomBarMenu(context), top: false),
      ],
    );
  }

  Widget baseBodyWidget({bool isChange = true}) {
    return isChange
        ? BaseBlocConsumerWidget<B>(
            isRetryCallback: isRetryCallback,
            retryCallback: () =>
                Future.delayed(Duration(seconds: 1), () => refreshPage()),
            bloc: bloc,
            listenWhen: listenerWhen,
            builder: baseCustomBody)
        : BaseBlocConsumerWidget<B>(
            isRetryCallback: isRetryCallback,
            retryCallback: () =>
                Future.delayed(Duration(seconds: 1), () => refreshPage()),
            bloc: bloc,
            listener: listener,
            builder: builder);
  }

  Widget baseCustomBody(BuildContext context, BaseState state) {
    isLoading = false;
    return Container(
        child: SmartRefresher(
      physics: BouncingScrollPhysics(),
      header: WaterDropMaterialHeader(
        color:
            isPrimary == true ? Colors.white : Theme.of(context).primaryColor,
        backgroundColor: isPrimary == true
            ? Theme.of(context).primaryColorLight
            : Colors.white,
      ),
      enablePullDown: true,
      key: _refresherKey,
      controller: _refreshCtr,
      onRefresh: () async {
        isLoading = true;
        await Future.delayed(Duration(milliseconds: 200), () {
          initBloc();
          _refreshCtr.refreshCompleted();
        });
      },
      child: builder(context, state),
    ));
  }

  @override
  initState() {
    bloc = inject<B>();
    initBloc();
    super.initState();
  }

  @override
  void callRefreshPage() {
    refreshPage();
  }

  @override
  dispose() {
    _refreshCtr.dispose();
    bloc?.close();
    super.dispose();
  }

  @protected
  Color? scaffoldBackgroundColor = AppColors.backgroundWhiteColor;

  @protected
  Widget appTopBarMenu(BuildContext context) => Container();

  @protected
  Widget appBottomBarMenu(BuildContext context) => Container();

  @protected
  void listener(BuildContext context, BaseState state) {}

  @protected
  bool listenerWhen(BaseState previous, BaseState state) => false;

  @protected
  Widget builder(BuildContext context, BaseState state);

  @protected
  void initBloc();

  @protected
  void refreshPage() {}

  @protected
  bool? isBottomBarThemeColor = false;

  @protected
  bool? isRefreshLoading = false;

  @protected
  bool? isFilter = false;

  @protected
  bool? isExit = false;

  @protected
  bool? isForceExit = false;

  @protected
  bool? isRetryCallback = false;

  @protected
  bool? isResizeToAvoidBottomInset = false;

  @protected
  bool? isFromHome = false;

  @protected
  bool? isPrimary = false;

  @protected
  bool? isWarning = false;

  void clearFocus() {
    FocusScopeNode currentFocus = FocusScope.of(context);
    if (!currentFocus.hasPrimaryFocus && currentFocus.focusedChild != null) {
      FocusManager.instance.primaryFocus?.unfocus();
    }
  }

  @override
  Future<void> onNotify(RemoteMessage notification) async {}
}
